@@ -8,11 +8,10 @@ use proj_sys::{
     proj_create, proj_create_crs_to_crs, proj_destroy, proj_errno_string, proj_get_area_of_use,
     proj_grid_cache_set_enable, proj_info, proj_normalize_for_visualization, proj_pj_info,
     proj_trans, proj_trans_array, PJconsts, PJ_AREA, PJ_CONTEXT, PJ_COORD, PJ_DIRECTION_PJ_FWD,
-    PJ_DIRECTION_PJ_INV, PJ_INFO, PJ_LP, PJ_XY,
+    PJ_DIRECTION_PJ_INV, PJ_INFO, PJ_LPZT, PJ_XYZT,
 };
 use std::{
-    convert,
-    ffi,
+    convert, ffi,
     fmt::{self, Debug},
     str,
 };
@@ -187,7 +186,7 @@ fn area_set_bbox(parea: *mut proj_sys::PJ_AREA, new_area: Option<Area>) {
 /// called by Proj::new and ProjBuilder::transform_new_crs
 fn transform_string(ctx: *mut PJ_CONTEXT, definition: &str) -> Result<Proj, ProjCreateError> {
     let c_definition =
-        CString::new(definition).map_err(|e| ProjCreateError::ArgumentNulError(e))?;
+        CString::new(definition).map_err(ProjCreateError::ArgumentNulError)?;
     let ptr = result_from_create(ctx, unsafe { proj_create(ctx, c_definition.as_ptr()) })
         .map_err(|e| ProjCreateError::ProjError(e.message(ctx)))?;
     Ok(Proj {
@@ -204,8 +203,8 @@ fn transform_epsg(
     to: &str,
     area: Option<Area>,
 ) -> Result<Proj, ProjCreateError> {
-    let from_c = CString::new(from).map_err(|e| ProjCreateError::ArgumentNulError(e))?;
-    let to_c = CString::new(to).map_err(|e| ProjCreateError::ArgumentNulError(e))?;
+    let from_c = CString::new(from).map_err(ProjCreateError::ArgumentNulError)?;
+    let to_c = CString::new(to).map_err(ProjCreateError::ArgumentNulError)?;
     let proj_area = unsafe { proj_area_create() };
     area_set_bbox(proj_area, area);
     let ptr = result_from_create(ctx, unsafe {
@@ -503,7 +502,7 @@ impl Proj {
     ///
     /// # Examples
     ///
-    /// Construcing a `Proj` from a PROJ string definition:
+    /// Constructing a `Proj` from a PROJ string definition:
     ///
     /// ```
     /// let transformer = proj::Proj::new(
@@ -737,11 +736,17 @@ impl Proj {
         // This signals that we wish to project geodetic coordinates.
         // For conversion (i.e. between projected coordinates) you should use
         // PJ_XY {x: , y: }
-        let coords = PJ_LP { lam: c_x, phi: c_y };
+        // We also initialize z and t in case libproj tries to read them.
+        let coords = PJ_LPZT {
+            lam: c_x,
+            phi: c_y,
+            z: 0.0,
+            t: f64::INFINITY,
+        };
         unsafe {
             proj_errno_reset(self.c_proj);
             // PJ_DIRECTION_* determines a forward or inverse projection
-            let trans = proj_trans(self.c_proj, inv, PJ_COORD { lp: coords });
+            let trans = proj_trans(self.c_proj, inv, PJ_COORD { lpzt: coords });
             // output of coordinates uses the PJ_XY struct
             new_x = trans.xy.x;
             new_y = trans.xy.y;
@@ -802,10 +807,19 @@ impl Proj {
         let new_x;
         let new_y;
         let err;
-        let coords = PJ_XY { x: c_x, y: c_y };
+
+        // This doesn't seem strictly correct, but if we set PJ_XY or PJ_LP here, the
+        // other two values remain uninitialized and we can't be sure that libproj
+        // doesn't try to read them. proj_trans_generic does the same thing.
+        let xyzt = PJ_XYZT {
+            x: c_x,
+            y: c_y,
+            z: 0.0,
+            t: f64::INFINITY,
+        };
         unsafe {
             proj_errno_reset(self.c_proj);
-            let trans = proj_trans(self.c_proj, PJ_DIRECTION_PJ_FWD, PJ_COORD { xy: coords });
+            let trans = proj_trans(self.c_proj, PJ_DIRECTION_PJ_FWD, PJ_COORD { xyzt });
             new_x = trans.xy.x;
             new_y = trans.xy.y;
             err = proj_errno(self.c_proj);
@@ -924,7 +938,12 @@ impl Proj {
                 let c_x: c_double = point.x().to_f64().ok_or(ProjError::FloatConversion)?;
                 let c_y: c_double = point.y().to_f64().ok_or(ProjError::FloatConversion)?;
                 Ok(PJ_COORD {
-                    xy: PJ_XY { x: c_x, y: c_y },
+                    xyzt: PJ_XYZT {
+                        x: c_x,
+                        y: c_y,
+                        z: 0.0,
+                        t: f64::INFINITY,
+                    },
                 })
             })
             .collect::<Result<Vec<_>, ProjError>>()?;
@@ -1085,22 +1104,22 @@ mod test {
         let to = "EPSG:4258";
 
         let mut online_builder = ProjBuilder::new();
-        let mut offline_builder = ProjBuilder::new();
+        let offline_builder = ProjBuilder::new();
 
-        assert_eq!(online_builder.network_enabled(), false);
-        assert_eq!(offline_builder.network_enabled(), false);
+        assert!(!online_builder.network_enabled());
+        assert!(!offline_builder.network_enabled());
 
         online_builder.enable_network(true).unwrap();
-        assert_eq!(online_builder.network_enabled(), true);
-        assert_eq!(offline_builder.network_enabled(), false);
+        assert!(online_builder.network_enabled());
+        assert!(!offline_builder.network_enabled());
 
         // Disable caching to ensure we're accessing the network.
         // Cache is stored in proj's [user writeable directory](https://proj.org/resource_files.html#user-writable-directory)
         online_builder.grid_cache_enable(false);
 
         // I expected the following call to trigger a download, but it doesn't!
-        let online_proj = online_builder.proj_known_crs(&from, &to, None).unwrap();
-        let offline_proj = offline_builder.proj_known_crs(&from, &to, None).unwrap();
+        let online_proj = online_builder.proj_known_crs(from, to, None).unwrap();
+        let offline_proj = offline_builder.proj_known_crs(from, to, None).unwrap();
 
         // download begins here:
         // File to download: uk_os_OSTN15_NTv2_OSGBtoETRS.tif
@@ -1160,7 +1179,7 @@ mod test {
         let ep = tf.get_url_endpoint().unwrap();
         assert_eq!(&ep, "https://cdn.proj.org");
         tf.set_url_endpoint("https://github.com/georust").unwrap();
-        let proj = tf.proj_known_crs(&from, &to, None).unwrap();
+        let proj = tf.proj_known_crs(from, to, None).unwrap();
         let ep = proj.get_url_endpoint().unwrap();
         // Has the new endpoint propagated to the Proj instance?
         assert_eq!(&ep, "https://github.com/georust");
@@ -1169,7 +1188,7 @@ mod test {
     fn test_from_crs() {
         let from = "EPSG:2230";
         let to = "EPSG:26946";
-        let proj = Proj::new_known_crs(&from, &to, None).unwrap();
+        let proj = Proj::new_known_crs(from, to, None).unwrap();
         let t = proj
             .convert(MyPoint::new(4760096.421921, 3744293.729449))
             .unwrap();
@@ -1255,11 +1274,11 @@ mod test {
     fn test_conversion() {
         // Generated by PROJ by specifying "from" and "to" EPSG codes and calling def()
         let projstring = "
-            proj=pipeline step proj=unitconvert xy_in=us-ft 
-            xy_out=m step inv proj=lcc lat_0=32.1666666666667 
-            lon_0=-116.25 lat_1=33.8833333333333 lat_2=32.7833333333333 
-            x_0=2000000.0001016 y_0=500000.0001016 ellps=GRS80 step proj=lcc lat_0=32.1666666666667 
-            lon_0=-116.25 lat_1=33.8833333333333 lat_2=32.7833333333333 x_0=2000000 y_0=500000 
+            proj=pipeline step proj=unitconvert xy_in=us-ft
+            xy_out=m step inv proj=lcc lat_0=32.1666666666667
+            lon_0=-116.25 lat_1=33.8833333333333 lat_2=32.7833333333333
+            x_0=2000000.0001016 y_0=500000.0001016 ellps=GRS80 step proj=lcc lat_0=32.1666666666667
+            lon_0=-116.25 lat_1=33.8833333333333 lat_2=32.7833333333333 x_0=2000000 y_0=500000
             ellps=GRS80
             ";
         let nad83_m = Proj::new(projstring).unwrap();
@@ -1332,7 +1351,7 @@ mod test {
     fn test_array_convert() {
         let from = "EPSG:2230";
         let to = "EPSG:26946";
-        let ft_to_m = Proj::new_known_crs(&from, &to, None).unwrap();
+        let ft_to_m = Proj::new_known_crs(from, to, None).unwrap();
         let mut v = vec![
             MyPoint::new(4760096.421921, 3744293.729449),
             MyPoint::new(4760197.421921, 3744394.729449),
@@ -1348,7 +1367,7 @@ mod test {
     fn test_input_order() {
         let from = "EPSG:4326";
         let to = "EPSG:2230";
-        let to_feet = Proj::new_known_crs(&from, &to, None).unwrap();
+        let to_feet = Proj::new_known_crs(from, to, None).unwrap();
         // 👽
         let usa_m = MyPoint::new(-115.797615, 37.2647978);
         let usa_ft = to_feet.convert(usa_m).unwrap();
